# Requests and responses message types

NaifJs is a backend engine independent from the frontend messaging channel 
(an instant messaging app like Telegram, Slack, etc. or a command line interface, anything). 

The main program, when using NaifJs, through a specific messaging adapter API, 
is in charge to get messages from the channel and send messages to the channel, 
using different media content types.


## Request message data 

Using NaifJs, the main node program is in charge to convert each incoming message from the channel, calling the dialog manager with `request` API.

NaifJs accepts requests data payload as pure texts or a javascript object data structures, 
containing at least a `text` attribute,(denoting a user written utterance), 
a `speech` attribute, denoting that user spoken the utterance, and more arbitrary optional parameters.

Examples:

- Written input, where data payload is a string):

  ```javascript
  naif.request(chatid, 'my name is Giorgio Robino' )
  ```

- Written input, where data payload is an object:

  ```javascript
  naif.request(chatid, {text: 'my name is Giorgio Robino'} )
  ```

- Speech (transcript from a speech-to-text engine), where data payload is an object:

  ```javascript
  naif.request(chatid, {text: 'my name is giorgio robino', speech: true} )
  ```


## Response message data

The same main program has to supply a `response` callback to send the message to the channel 
(see the Telegram bot pseudo-code example here above).
The output message (reply to user) could be a flat text (attribute of DSL statement `response`) 
or a specific javascript object data structure. 

Here below some examples with different kind of `response` data payloads:

- text only output:

  ```javascript
  naif.response(chatid, { text: 'my name is Giorgio Robino' } )
  ```

- speech with text caption:

  ```javascript
  naif.response(chatid, { text: 'my name is Giorgio Robino', speech: true, caption: 'my name is Giorgio Robino' } )
  ```

- image with text caption and voice over:

  ```javascript
  naif.response(chatid, { 
    image: 'path/to/image.jpg', 
    text: 'my name is Giorgio Robino', 
    speech: true, 
    caption: 'my name is Giorgio Robino' 
  } )
  ```


- earcon:

  ```javascript
  naif.response(chatid, { earcon: 'path/to/earcon.ogg', text: false, } )
  ```


## Response timeout handlers (no answer management)

A dialog agent reply or prompt to user with a `response` API statement. 
By default there is no any timeout management: 

```javascript
say('Hello! What\'s your name?')
```

In the case above the dialog is stuck, waiting a reply from user.

NaifJs is able to manage the user silence (no answer) after a prompt.
This is done passing an optional object to the `response` function:

```javascript
say('Hello! What\'s your name?', { callback: askNameTimeout, milliseconds: DELAY_15S })
```

The timeout callback is triggered after a certain number of specified milliseconds and usually just reprompt the user. 
In the example here below, the handler reprompt twice, afterward give-up. 

```javascript
// output state handler, dialog entry point (start) 
function askName() {
    
  say('Hello! What\'s your name?', askNameTimeout)

  next(getNameState)
}

// input state handler 
function getName(sentence) {

  // create a regexp rule to get a first name (e.g. Anna Lisa, Giorgio La Malfa)
  const firstNameReg = '(?<firstName>[A-Z][a-z]{1,}(\\s[A-Z][a-z]{1,})*)'
  const introNameReg = '^(my name is |name me |I\'m )?'
  const FIRSTNAME_REGEX = new RegExp ( introNameReg + firstNameReg ) 

  // delay for the first timeout
  const DELAY_15S = 15 * 1000

  // the function askNameTimeout is called 15 secs after the say statement 
  const timeout = { 
    callback: askNameTimeout, 
    milliseconds: DELAY_15S 
  }

  switch ( take(sentence) ) {
 
    // pattern 1
    case match( /hi|hello|good morning|good afternoon/i ):
      
      say( 'Hi there! What\'s your name?', timeout )
      break

    // pattern 2
    case match( FIRSTNAME_REGEX ):

      say( `Nice to meet you, ${slots().firstName}!` )
      end()
      break

    // fallback pattern
    default:

      say( 'I do not understand.' )
      say( 'What is your name?', timeout )
      break
  }
}


/**
 * ask name timeout handler
 *
 * @param {string} user id
 * @param {Object} params
 */ 
function askNameTimeout(userid, params) {

  const DELAY_30S = 30 * 1000
  const DELAY_60S = 45 * 1000

  setChat(userid)

  switch( getCounterTimeout() ) {

    case 1:
      // first timeout goes off
      say('What\'s your name?')

      // reschedule a timeout in 30 seconds
      startTimeout(askNameTimeout, DELAY_30S)
      break

    case 2:
      // second timeout
      say('Are you there? Please tell me what is your name')

      // reschedule a timeout in 60 seconds
      startTimeout(askNameTimeout, DELAY_60S)
      break

    case 3:
      // third timeout: close the conversation
      stopTimeout()
      end()
   }   
 }   
```

The running example is the [here](../examples/getUserNameTimeout/getUserName.js).

Please note that NaifJs allows developer to decide how to do when timeout is up.
In the example above the user is re-prompted twice, he's asked to reply, afterward  the dialogue end.
This is an arbitrary policy. 
Another option could be to let the dialog move to another state, 
maybe reformulating the question or moving to another topic.

---

[top](#) | [home](../README.md)
